//     EEPROM Management functions of RDS USB software 
//
// =====================================================
//  Description
//    This file include initialisation and management functions
//       of EEPROM Management 
//    This functions calling avr-libc library functions  :
//       uint8_t eeprom_read_byte (const uint8_t *addr)
//       uint16_t eeprom_read_word (const uint16_t *addr)
//       void eeprom_write_byte (uint8_t *addr, uint8_t val)
//       void eeprom_write_word (uint16_t *addr, uint16_t val)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
//=======================================
// Author : G.Laroche www.g.laroche.free.fr
//=======================================
// History:
// 19/05/07    V1.0
//       creation 
// 11/06/07    V1.1
//       adding comment on unused function to limit binary size for ATmega8 version compatibility 
// 18/07/07    V1.2
//       adding AF init to default value missing in function  EEP_SetDefault()
//=======================================


#include "RDS_EepromMgt.h"
#include "RDS_RS232.h"  /* for debug functionnalities */

/* include system */
#include <avr/io.h>
#include <stdlib.h>
#include <avr/pgmspace.h>   

/* static variable  */
static  T_RDS_EEPROM_data  * pt_EEPROM_Data;  /* pointer on EEPROM data structure */





/* EEPROM Management function */


/* pointer initialisation , validity control and default initialisation if no valid data */
void EEP_Init(void)
{
    char ResultCheck;    /* checking EEPROM result  */
	
    /* init pointer on data structure at end of EEPROM memory  delimited by E2END */
    pt_EEPROM_Data = (T_RDS_EEPROM_data  *) (E2END - sizeof( T_RDS_EEPROM_data));
	
	/* validity EEPROM control  */
	ResultCheck = EEP_Check();

	/* IF data are not valid */
	/* THEN */
	if (ResultCheck != 0)
	{
	    /* default EERPOM init */
		EEP_SetDefault();
	/* ENDIF */
	}
	

 
}


/* default EEPROM initialisation */
void EEP_SetDefault(void)
{
    unsigned char Index;
    unsigned char IndexType;
    
    PGM_P ptStringPGM;       /* pointer to string in PGM space for default value init */
    
//    uint8_t TempPS[RDS_SIZE_PSNAME]=EEP_PSNAME_DEF_VAL;
    
   	eeprom_write_word(&(pt_EEPROM_Data->PICode),EEP_PI_DEF_VAL);
   	eeprom_write_byte(&(pt_EEPROM_Data->PTY),EEP_PTY_DEF_VAL);
   	eeprom_write_byte(&(pt_EEPROM_Data->TP),EEP_TP_DEF_VAL);
   	eeprom_write_byte(&(pt_EEPROM_Data->TA),EEP_TA_DEF_VAL);
   	eeprom_write_byte(&(pt_EEPROM_Data->DI),EEP_DI_DEF_VAL);
   	eeprom_write_byte(&(pt_EEPROM_Data->MS),EEP_MS_DEF_VAL);
   	eeprom_write_byte(&(pt_EEPROM_Data->NbAF),EEP_NBAF_DEF_VAL);
    /* init AF default value */
   	for(Index=0; Index!=RDS_SIZE_AFTAB;Index++)
   	{
   	      	eeprom_write_byte(&(pt_EEPROM_Data->AFTab[Index]),EEP_AF_DEF_VAL - RDS_AF_FIRST_AF_FRQ);         
   	      	ptStringPGM++;
    }


   	/* init pointer to PSNAME */
   	ptStringPGM = PSTR(EEP_PSNAME_DEF_VAL);
   	   	
   	for(Index=0; Index!=RDS_SIZE_PSNAME;Index++)
   	{
   	      	eeprom_write_byte(&(pt_EEPROM_Data->PSName[Index]),pgm_read_byte(ptStringPGM));         
   	      	ptStringPGM++;
    }
   	/* init pointer to PSNAME */
   	ptStringPGM = PSTR(EEP_RTXT_DEF_VAL);
   	Index=0;
   	do 
   	{
          IndexType=pgm_read_byte(ptStringPGM);
          eeprom_write_byte(&(pt_EEPROM_Data->RadioTxt[Index]),IndexType);    
          Index++;
          ptStringPGM++;
    }
    while ((Index<RDS_SIZE_RADIOTEXT) && (IndexType != '\r'));
    
    /* Fill unused data to end */
    while (Index<RDS_SIZE_RADIOTEXT) 
    {
          eeprom_write_byte(&(pt_EEPROM_Data->RadioTxt[Index]),'\r');    
          Index++;
          ptStringPGM++;    
    }

    for (IndexType=0; IndexType!= RDS_SIZE_TYPE_GROUPS; IndexType++)
    {
        for(Index=0; Index!=RDS_SIZE_GROUPS; Index++)
        {
   	      	eeprom_write_byte(&(pt_EEPROM_Data->GroupRate[IndexType][Index]),0);             
        }
    }
   	eeprom_write_byte(&(pt_EEPROM_Data->GroupRate[0][0]),100);   /* 100% group0A by default */            
   	eeprom_write_byte(&(pt_EEPROM_Data->GroupRate[(RDS_GROUP_TYPE_2A & RDS_GROUP_TYPE_SELECT)][(RDS_GROUP_TYPE_2A>>RDS_GROUP_TYPE_SHIFT)]),EEP_GROUPRATE_2A_DEF_VAL);   /* 2A group rate default value  */            
    
//  comment following line for ATmega8 compatibility (binary size)    
  	eeprom_write_byte(&(pt_EEPROM_Data->CoderAdress),EEP_CODER_ADDR_DEF_VAL);
  	eeprom_write_word(&(pt_EEPROM_Data->SiteAdress),EEP_SITE__ADDR_DEF_VAL);
   
	eeprom_write_word(&(pt_EEPROM_Data->Passwd),EEP_PASSWD_DEF_VAL);
   	eeprom_write_byte(&(pt_EEPROM_Data->StereoMode),EEP_STEREO_MODE_DEF_VAL);

	/* compute validity checksum */
	EEP_update_valid();   
}

/* EEPROM validity control, return 0 if OK,else  -1  */
char EEP_Check(void)
{ 
    char Result;
    unsigned int TmpCheckSum;  /* temporary check sum  value */
    unsigned int Index;        /* loop index for Checksum compute */	
    uint8_t * pt_EEP_data;      /* temporary pointer on EEPROM data */
	


	Result=0;   /*  OK by default */
	TmpCheckSum=0;
	pt_EEP_data = (uint8_t *) pt_EEPROM_Data;
	
	/* Compute chechsum loop without including validity control value  */
	for (Index = (sizeof(T_RDS_EEPROM_data) -sizeof(uint16_t));Index !=0; Index--)
	{
	     TmpCheckSum += eeprom_read_byte(pt_EEP_data);
		 pt_EEP_data++;
	}
	
	if ( TmpCheckSum != eeprom_read_word(&(pt_EEPROM_Data->Validity_EEPROM)))
	{
	   Result =-1;
	}
	return Result;
}


/* set new PI */
void EEP_set_PICode(uint16_t NewPICode)
{
    eeprom_write_word(&(pt_EEPROM_Data->PICode),NewPICode);
    /* compute validity data */
    EEP_update_valid();   /* in this case return value is not used */
}           

/* set new PTY */
void EEP_set_PTY(uint8_t NewPTY)
{
    eeprom_write_byte(&(pt_EEPROM_Data->PTY),NewPTY);
    /* compute validity data */
    EEP_update_valid();   /* in this case return value is not used */
}           

/* set new PSNAME */
void EEP_set_PSName(uint8_t NewPSName[RDS_SIZE_PSNAME])
{
    uint8_t Index;
    for (Index=0; Index!=RDS_SIZE_PSNAME;Index++)
    {
        eeprom_write_byte(&(pt_EEPROM_Data->PSName[Index]),NewPSName[Index]);
    }
    /* compute validity data */
    EEP_update_valid();   /* in this case return value is not used */
}    
       
/* set new TP */
void EEP_set_TP(uint8_t NewTP)
{
    eeprom_write_byte(&(pt_EEPROM_Data->TP),NewTP);
    /* compute validity data */
    EEP_update_valid();   /* in this case return value is not used */
} 
/* set new TA */
void EEP_set_TA(uint8_t NewTA)
{
    eeprom_write_byte(&(pt_EEPROM_Data->TA),NewTA);
    /* compute validity data */
    EEP_update_valid();   /* in this case return value is not used */
}          
/* set new DI */
void EEP_set_DI(uint8_t NewDI)
{
    eeprom_write_byte(&(pt_EEPROM_Data->DI),NewDI);
    /* compute validity data */
    EEP_update_valid();   /* in this case return value is not used */
}           
/* set new MS */
void EEP_set_MS(uint8_t NewMS)
{
    eeprom_write_byte(&(pt_EEPROM_Data->MS),NewMS);
    /* compute validity data */
    EEP_update_valid();   /* in this case return value is not used */
}           
/* set new NbAF */
void EEP_set_NbAF(uint8_t NewNbAF)
{
    eeprom_write_byte(&(pt_EEPROM_Data->NbAF),NewNbAF);
    /* compute validity data */
    EEP_update_valid();   /* in this case return value is not used */
}           
/* set new AF */
void EEP_set_AF(uint8_t NewNbAF,uint8_t Index)
{
    eeprom_write_byte(&(pt_EEPROM_Data->AFTab[Index]),NewNbAF);
    /* compute validity data */
    EEP_update_valid();   /* in this case return value is not used */
}          
/* set new RadioText  */
void EEP_set_RadioText(uint8_t NewRT[RDS_SIZE_RADIOTEXT])
{
    uint8_t Index;
    for (Index=0; Index!=RDS_SIZE_RADIOTEXT;Index++)
    {
        eeprom_write_byte(&(pt_EEPROM_Data->RadioTxt[Index]),NewRT[Index]);
    }
    /* compute validity data */
    EEP_update_valid();   /* in this case return value is not used */
}    
           
/* set new group rate */
void EEP_set_GroupRate(uint8_t NewGroupRate,uint8_t IndexType, uint8_t IndexGroup)
{
    eeprom_write_byte(&(pt_EEPROM_Data->GroupRate[IndexType][IndexGroup]),NewGroupRate);
    /* compute validity data */
    EEP_update_valid();   /* in this case return value is not used */
}


//  comment following line for ATmega8 compatibility (binary size)    
//           
/* set new CoderAdress */
void EEP_set_CoderAdress(uint8_t NewCoderAdress)
{
   eeprom_write_byte(&(pt_EEPROM_Data->CoderAdress),NewCoderAdress);
   /* compute validity data */
   EEP_update_valid();   /* in this case return value is not used */
}
           
/* set new SiteAdress */
void EEP_set_SiteAdress(uint16_t NewSiteAdress){
   eeprom_write_word(&(pt_EEPROM_Data->SiteAdress),NewSiteAdress);
   /* compute validity data */
   EEP_update_valid();   /* in this case return value is not used */
}
          

/* store new Passwd */
void EEP_set_Passwd(uint16_t NewPasswd)
{
   eeprom_write_word(&(pt_EEPROM_Data->Passwd),NewPasswd);
   /* compute validity data */
   EEP_update_valid();   /* in this case return value is not used */

}
/* set new StereoMode */
void EEP_set_StereoMode(uint8_t NewStereoMode)
{
    eeprom_write_byte(&(pt_EEPROM_Data->StereoMode),NewStereoMode);
    /* compute validity data */
    EEP_update_valid();   /* in this case return value is not used */
}           





/* return current PICode */
uint16_t EEP_get_PICode(void)
{
   return (eeprom_read_word(&(pt_EEPROM_Data->PICode)));
}               

/* return current PTY */
uint8_t EEP_get_PTY(void) 
{
   return (eeprom_read_byte(&(pt_EEPROM_Data->PTY)));
}
               

/* return current PSNAme[Index]] */
unsigned char EEP_get_PSName(uint8_t Index)               
{
   return (eeprom_read_byte(&(pt_EEPROM_Data->PSName[Index])));
}               

/* return current TP */
uint8_t EEP_get_TP(void)
{
   return (eeprom_read_byte(&(pt_EEPROM_Data->TP)));
}               

/* return current TA */
uint8_t EEP_get_TA(void)
{
   return (eeprom_read_byte(&(pt_EEPROM_Data->TA)));
}               

/* return current DI */
uint8_t EEP_get_DI(void)
{
   return (eeprom_read_byte(&(pt_EEPROM_Data->DI)));
}               

/* return current MS */
uint8_t EEP_get_MS(void)
{
   return (eeprom_read_byte(&(pt_EEPROM_Data->MS)));
}               

/* return current NbAF */
uint8_t EEP_get_NbAF(void)
{
   return (eeprom_read_byte(&(pt_EEPROM_Data->NbAF)));
}               

/* return current AF[Index] */
uint8_t EEP_get_AF(uint8_t Index)
{
   return (eeprom_read_byte(&(pt_EEPROM_Data->AFTab[Index])));
}               

/* return current radiotext[Index] */
unsigned char EEP_get_Radiotext(uint8_t Index)
{
   return (eeprom_read_byte(&(pt_EEPROM_Data->RadioTxt[Index])));
}               

/* return current GroupRate[IndexType,IndexGroup] */
uint8_t EEP_get_GroupRate(uint8_t IndexType,uint8_t IndexGroup)
{
   return (eeprom_read_byte(&(pt_EEPROM_Data->GroupRate[IndexType][IndexGroup])));
}               

//  comment following line for ATmega8 compatibility (binary size)    
/* return current CoderAdress */
uint8_t EEP_get_CoderAdress(void)
{
  return (eeprom_read_byte(&(pt_EEPROM_Data->CoderAdress)));
}               

/* return current SiteAdress */
uint16_t EEP_get_SiteAdress(void)
{
   return (eeprom_read_word(&(pt_EEPROM_Data->SiteAdress)));
}               

/* return passwd */
uint16_t EEP_get_Passwd(void)
{
   return (eeprom_read_word(&(pt_EEPROM_Data->Passwd)));
}
/* return current configuration StereoMode */
uint8_t EEP_get_StereoMode(void)
{
   return (eeprom_read_byte(&(pt_EEPROM_Data->StereoMode)));
}               
              






/* compute checksum validity,store and return value  */
uint16_t EEP_update_valid(void)
{
    uint16_t TmpCheckSum;      /* temporary checksum value */
    unsigned int Index;        /* loop index for checksum computing  */	
    uint8_t * pt_EEP_data;     /* temporary pointer on EEPROM data */
	
	
    TmpCheckSum=0;
    pt_EEP_data = (uint8_t *) pt_EEPROM_Data;
	
    /* checksum computing loop without including control validity result */
    for (Index = (sizeof(T_RDS_EEPROM_data) -sizeof(uint16_t));Index !=0; Index--)
    {
         TmpCheckSum += eeprom_read_byte(pt_EEP_data);
         pt_EEP_data++;
    }
    /* update checksum result */
    eeprom_write_word(&(pt_EEPROM_Data->Validity_EEPROM),TmpCheckSum);

    return (TmpCheckSum);
}







