//     RS232 Management functions of RDS software 
//
// =====================================================
//  Description
//    This file include initialisation and management functions
//       of RS232  Management 
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
//=======================================
// Author : G.Laroche www.g.laroche.free.fr
//=======================================
// History:
// 19/05/07    V1.0
//       create 
//       derivative from PFM ATmega8 board  :
//           TIPLL_USART2.c  12/2006    V1.2 author g.Laroche
// 11/06/07    V1.1
//       comment unused function to limit binary size for ATmega8 version compatibility 
//=======================================


/* Includes */
#include "RDS_RS232.h"
#include "RDS_EepromMGt.h" 
#include "RDS.h" 
#include "RDS_ClockDriver.h"
#include "RDS_IOMgt.h"

#include <avr/interrupt.h>
#include <stdlib.h>
#include <avr/io.h>
#include <string.h>

/* pgmspace.h is necessary to use char string in flash memory */
/* PSTR("string")  to direct flash using, else String are duplicate into RAM  !!!! */
#include <avr/pgmspace.h>   

/* local constantes definitions */
#define RS_TAILLE_STRING_PARAM_UI 10
#define RS_TAILLE_STRING_CONVERT  20



/* extern variables */
extern unsigned char CurrentPSName[RDS_SIZE_PSNAME];        /* current PS Name */
extern unsigned char CurrentRTxt[RDS_SIZE_RADIOTEXT];       /* current RadioText ended by 0x0d '\r' , unused data should be set to \r for optimisation */
extern unsigned char CurrentTxtAB;                          /* current Text A/B , each change clear line text */
extern unsigned short CurrentTP;                            /* current TP */
extern unsigned short CurrentPTY;                           /* current PTY */
extern unsigned char CurrentTA;                             /* current TA */
extern unsigned char CurrentMS;                             /* current MS */
extern unsigned char CurrentDI;                             /* current DI */

extern unsigned char RDSDumpBuf;                            /* Dump buffer RDS */


/* Static Variables */
static unsigned char RS_RxBuf[RS_RX_BUFFER_SIZE];
static volatile unsigned char RS_RxHead;
static volatile unsigned char RS_RxTail;
static unsigned char RS_TxBuf[RS_TX_BUFFER_SIZE];

static volatile unsigned char RS_TxHead;
static volatile unsigned char RS_TxTail;

static unsigned char RS_NbCmd;   /* Command counter in receive buffer */
static unsigned char RS_TmpConvertString[RS_TAILLE_STRING_CONVERT];  /* ascii conversion */

static char SyntaxError;  /* !=0 si erreur de syntaxe */
static unsigned char PreviousCommand;       /* Previous command */
static unsigned char CurrentCommand;       /* Current command */



/* reuse this frequently using string,to reduce flash using size */
//static char NewLineKHz[]="0 KHz\r\n";

/***************************************************/
/*    RS_Init :                                    */
/*    --------                                     */
/*      initialize RS232 link                      */
/***************************************************/
/* Initialize RS */
void RS_Init( unsigned int baudrate )
{
	unsigned char x;


	/* Set the baud rate */
	UBRRH = (unsigned char) (baudrate>>8);                  
	UBRRL = (unsigned char) baudrate;
	
	/* Enable UART receiver and transmitter */
	UCSRB = ( ( 1 << RXEN ) | ( 1 << TXEN ) ); 
	
	/* Set frame format: 8 data 2stop */
	/* URSEL positionne a 1 pour selectionner UCSRC (@ commune avec UBRRH) */
	UCSRC = (1<<URSEL)|(1<<USBS)|(1<<UCSZ1)|(1<<UCSZ0);  
	
	/* Flush receive buffer */
	x = 0; 			    

	RS_RxTail = x;
	RS_RxHead = x;
	RS_TxTail = x;
    RS_TxHead = x;
    RS_NbCmd = x;

    /* enable interrupt TX and RX */
    // UCSRB |=  ( 1 << RXCIE ) | (1<< TXCIE);
    UCSRB |=  ( 1 << RXCIE )    ;
    
    /* init static variable */
    PreviousCommand=' ';


    
    
}

/***************************************************/
/***************************************************/
/*           low level functions                   */
/***************************************************/
/***************************************************/
/***************************************************/


/* Interrupt handlers */
/* Receive interrupt handler */
/* SIGNAL don't allowing new IT before end processing */
SIGNAL(SIG_UART_RECV)
{
	unsigned char data;
	unsigned char tmphead;

//RS_DebugPrintfc('R');


	/* Read the received data */
	data = UDR;                 
	/* Calculate buffer index */
	tmphead = ( RS_RxHead + 1 ) & RS_RX_BUFFER_MASK;
	RS_RxHead = tmphead;      /* Store new index */

	if ( tmphead == RS_RxTail )
	{
		/* ERROR! Receive buffer overflow */
	        RS_DebugPrintfc( '!' ); /* send error character */
	}
	else
	{
	    /* using Debug function (polling) to avoid dead lock if transmit buffer is full */
	    RS_DebugPrintfc( data ); /* Echo the received character */
    }
	
	RS_RxBuf[tmphead] = data; /* Store received data in buffer */
	
	if (data == '\r')
	{
	   /* if 'CR' then incrementing command number */
	   RS_NbCmd++;	
	}
}

// SIGNAL(SIG_UART_TRANS)
// {
// 	
// RS_DebugPrintfc('T');
// }

/*  WARNING for USB driver compatibility USING INTERRUPT and no SIGNAL !!!!!! */
SIGNAL(SIG_UART_DATA)
{
	unsigned char tmptail;
	
// RS_DebugPrintfc('D');


	/* Check if all data is transmitted */
	if ( RS_TxHead != RS_TxTail )
	{
		/* Calculate buffer index */
		tmptail = ( RS_TxTail + 1 ) & RS_TX_BUFFER_MASK;
		RS_TxTail = tmptail;      /* Store new index */
	
		UDR = RS_TxBuf[tmptail];  /* Start transmition */
	}
	else
	{
		UCSRB &= ~(1<<UDRIE);         /* Disable UDRE interrupt */
	}
}

/* Read and write functions low level */
/***************************************************/
/*    RS_Receive :                                 */
/*    ----------                                   */
/*      blocking read of a receive char in the     */
/*      receive buffer                             */
/*                                                 */
/***************************************************/
unsigned char RS_Receive( void )
{
	unsigned char tmptail;
	
	while ( RS_RxHead == RS_RxTail );  /* Wait for incomming data */
	tmptail = ( RS_RxTail + 1 ) & RS_RX_BUFFER_MASK;/* Calculate buffer index */
	RS_RxTail = tmptail;                /* Store new index */
	return RS_RxBuf[tmptail];           /* Return data */
}

/***************************************************/
/*    RS_Transmit :                                */
/*    ----------                                   */
/*      transmiting 1 char with buffer store and IT*/
/*                                                 */
/***************************************************/
void RS_Transmit( unsigned char data )
{
	unsigned char tmphead;



	/* Calculate buffer index */
	tmphead = ( RS_TxHead + 1 ) & RS_TX_BUFFER_MASK; /* Wait for free space in buffer */
	while ( tmphead == RS_TxTail )
	{
       /* debug !!!! toggle TALED */
       RDSIO_ToggleTALed();
    }

	RS_TxBuf[tmphead] = data;           /* Store data in buffer */
	RS_TxHead = tmphead;                /* Store new index */

	UCSRB |= (1<<UDRIE);                     /* Enable UDRE interrupt */

}


/***************************************************/
/*    DataInReceiveBuffer:                         */
/*    ---------------                              */
/*      Return 0 if no char in receive buffer      */
/*                                                 */
/***************************************************/
unsigned char DataInReceiveBuffer( void )
{
	return ( RS_RxHead != RS_RxTail ); /* Return 0 (FALSE) if the receive buffer is empty */
}

/***************************************************/
/***************************************************/
/*          Applicatives Fonctions                 */
/***************************************************/
/***************************************************/
/***************************************************/



/***************************************************/
/*    RS_ReadCmdChar:                              */
/*    ---------------                              */
/*      Return next char from receive buffer       */
/*      including command counter management       */
/*                                                 */
/***************************************************/
unsigned char RS_ReadCmdChar(void)
{
     unsigned char TmpChar;
     
     TmpChar=RS_Receive();
     
     if (TmpChar == '\r')
     {
           RS_decNbCmd(); /* decrementing command counter */     	
     }
     return(TmpChar);
}

/***************************************************/
/*    RS_ReadCmdParamUi:                           */
/*    ---------------                              */
/*     read an unsigned integer parameter delimited*/
/*     by ' ' or '\r' and return it after converted*/
/*                                                 */
/***************************************************/
unsigned int  RS_ReadCmdParamUi(void)
{
	char TmpConvString[RS_TAILLE_STRING_PARAM_UI];
	
	/* read string and convert ascii to integer */
	RS_ReadCmdParamStr(TmpConvString,RS_TAILLE_STRING_PARAM_UI);
	return (atoi(TmpConvString));
}




/***************************************************/
/*    RS_ReadCmdParamStr:                          */
/*    ---------------                              */
/*      read a string parameter delimited by       */
/*      ' ' or '\r' and return it                  */
/*      the MaxRead parameter define max size      */
/*      including '\0'                             */
/*                                                 */
/***************************************************/

void     RS_ReadCmdParamStr(char * ParamReadOut,unsigned char MaxRead)
{
     unsigned char TmpChar;   // tempoprary reading variable 
     unsigned char TmpMaxRead; // max size of string 
     
	 
     TmpChar=RS_ReadCmdChar();
     TmpMaxRead= MaxRead-1;  // dec to let free space for ending \0
     /* suppress first separating */
     while (TmpChar == ' ')
     {
         TmpChar=RS_ReadCmdChar();
     }
     while ( (TmpChar != ' ') && (TmpChar != '\r') && (TmpMaxRead !=0) )
     {
     	*ParamReadOut=TmpChar;
     	ParamReadOut++;
        TmpChar=RS_ReadCmdChar();
        TmpMaxRead--;
     }
     *ParamReadOut=0;   /* ending string */
}


/***************************************************/
/*    RS_decNbCmd :                                */
/*    -----------                                  */
/*      decrementing command counter               */
/*                                                 */
/***************************************************/
void RS_decNbCmd(void)
{
        cli();  /* disable Interrupt for critical section */
        RS_NbCmd--;
        sei();  /* enable interrupt , end of critical section */
}


/***************************************************/
/*    RS_getNbCmd :                                */
/*    -----------                                  */
/*      return command counter                     */
/*                                                 */
/***************************************************/
unsigned char RS_getNbCmd(void)
{
	return(RS_NbCmd);
}





/***************************************************/
/*    RS_PrintfStrPGM :                            */
/*    ---------------                              */
/*      send a string from flash memory without    */
/*      RAM copy                                   */
/*                                                 */
/***************************************************/
void RS_PrintfStrPGM(PGM_P StringChar )
{

        /*send a string to serial link */
        while (pgm_read_byte(StringChar) !='\0')
        {
            RS_Printfc(pgm_read_byte(StringChar) ); /* send a char  */
            StringChar++; /* incrementing pointer */
        };
}

/***************************************************/
/*    RS_PrintfStr :                               */
/*    -----------                                  */
/*      send a string from RAM memory              */
/*                                                 */
/***************************************************/
void RS_PrintfStr( unsigned char  * StringChar )
{

        /*send a string to serial link */
        while (*StringChar !='\0')
        {
            RS_Printfc( *StringChar ); /* send a char  */
            StringChar++; /* incrementing pointer */
        };
}

/***************************************************/
/*    RS_Printfc :                                 */
/*    -----------                                  */
/*      send a char reading with IT handler        */
/*                                                 */
/***************************************************/
void RS_Printfc( unsigned char  Caractere )
{
        /*send a char  to serial link */
        RS_Transmit( Caractere ); /* send a char */
}

/***************************************************/
/*    RS_PrintfUi :                                */
/*    -----------                                  */
/*      Send an unsigned integer converted into    */
/*      decimal if Format=0 or hexa if Format=1    */
/*                                                 */
/***************************************************/
void RS_PrintfUi( unsigned int  Data, unsigned char Format )
{
    /* printf of an unsigned int to the serial link with IT handler according to the following format  : 0= 10, 1=hexa */

	
    switch (Format)
    {
      case 1:
       {
       	  RS_PrintfStr(PSTR("0x"));
       	  utoa(Data,RS_TmpConvertString,16);
       }
       break;
       case 0:
       default:
       {
       	  utoa(Data,RS_TmpConvertString,10);
       }
    }
    RS_PrintfStr(RS_TmpConvertString);
}


/***************************************************/
/*    RS_ReceiveMgt:                               */
/*    -------------                                */
/*      Receive management                         */
/*                                                 */
/*                                                 */
/***************************************************/
void RS_ReceiveMgt( void )
{
    unsigned char CurrentCar;       /* caractere courant */
    unsigned char IndexParam;
    unsigned short TmpParam;
    unsigned char ParamCommand[3];   /* parameter 3 char + \0 */
    
    	
    
    /* test if a complete command recived else no processing*/
    if ( RS_getNbCmd() !=0)
    {
	/* Command process */
	/* Type of command format : "command [parameter1] [parameter2] return" */
	/* command correspond to 1 char : exple : F, S, P, ...*/
	/* Parameter is a number or string  */
	/* return is the char "\r" */
	
		SyntaxError=-1;                    /* SyntaxError =-1 --> parameter error */
                                           /* SyntaxError =0  --> command OK */
                                           /* SyntaxError =1  --> Command error or syntax error */
        /* switch 1st char readed */
        CurrentCar=RS_ReadCmdChar();
        switch (CurrentCar)
        {

          	
         	case 'S':
         	case 's':
         	case 'R':
         	case 'r':
         	{
                 
                 CurrentCommand=CurrentCar;
                 if(CurrentCommand=='r') CurrentCommand='R';
                 if(CurrentCommand=='s') CurrentCommand='S';
                 RS_ReadCmdParamStr(ParamCommand,3);
                 /* convert to uppercase */
                 strupr(ParamCommand);
                 
                 /* test "PS" */
                 /* --------- */  
                 if (strcmp_P(ParamCommand,PSTR("PS"))==0)
                 {
                     /* modifying PSNAME */
                     IndexParam=0;
                     /* read first delimiter */
                     CurrentCar=RS_ReadCmdChar();
                      while(   (IndexParam<RDS_SIZE_PSNAME) 
                           && (CurrentCar!='>')
                           && (CurrentCar!='\r') )
                     {
                         /* read next char  */
                         CurrentCar=RS_ReadCmdChar();                        
                         CurrentPSName[IndexParam]=CurrentCar;
                         IndexParam++;
                     }
                     if (IndexParam==RDS_SIZE_PSNAME)
                     {
                         SyntaxError=0; /* command OK */
                         if (CurrentCommand=='S')
                         {
                             /* store PSNAME in EEPROM */
                             EEP_set_PSName(CurrentPSName);
                         }
                     }
                     else
                     {
                         SyntaxError=1;
                     }
                 }  /* Endif PSNAME processing */
                 
        	     /* ------------------------------------------- */
                 
                 /* test "RT" */
                 /* --------- */  
                 if (strcmp_P(ParamCommand,PSTR("RT"))==0)
                 {
                     /* modifying RText max 64 char including \r , only 63 char available */
                     IndexParam=0;
                     /* read first delimiter */
                     CurrentCar=RS_ReadCmdChar();
                     while(   (IndexParam<RDS_SIZE_RADIOTEXT) 
                           && (CurrentCar!='>')
                           && (CurrentCar!='\r') )
                     {
                         /* read next char  */
                         CurrentCar=RS_ReadCmdChar();
                         CurrentRTxt[IndexParam]=CurrentCar;
                         IndexParam++;
                     }
                     /* if end loop equal to end delimiter or max size */
                     if  ((CurrentCar=='>') || (IndexParam == RDS_SIZE_RADIOTEXT))
                     {
                         /*replace last delimiter by '\r' */
                         /* and fill end arry by '\r' */
                         IndexParam--;
                         while (IndexParam <RDS_SIZE_RADIOTEXT)
                         {
                            CurrentRTxt[IndexParam]='\r';
                            IndexParam++;
                         }
                         SyntaxError=0; /* command OK */                         
                         if (CurrentCommand=='S')
                         {
                             /* store radiotext in EEPROM */
                             EEP_set_RadioText(CurrentRTxt);
                         }
                     }
                     else
                     {
                         SyntaxError=1;
                     }
                 }  /* Endif RT processing */
                 
                 if (strcmp_P(ParamCommand,PSTR("PY"))==0)
                 {
                       /* read PTY */
                       TmpParam=RS_ReadCmdParamUi();
                       TmpParam &=0x1F; /* only 5bits lenght for PTY */
                       CurrentPTY=  TmpParam<< 5; /* PTY should be shifted before used by RDS_GroupGenerate */
                       SyntaxError=0; /* command OK */                         
                       if (CurrentCommand=='S')
                       {
                          /* Store PTY in EEPROM */
                          EEP_set_PTY(TmpParam);
                       }                    
                 }
                 
                 if (strcmp_P(ParamCommand,PSTR("TP"))==0)
                 {
                       /* read TP */
                       TmpParam=RS_ReadCmdParamUi();
                       TmpParam &= 0x01;          /* only 1 bit lenght for TP field */
                       CurrentTP=  TmpParam<< 10;  /* TP should be shifted before used by RDS_GroupGenerate */ 
                       SyntaxError=0; /* command OK */                         
                       if (CurrentCommand=='S')
                       {
                          /* Store TP in EEPROM */
                          EEP_set_TP(TmpParam);
                       }                    
                 }   
                 if (strcmp_P(ParamCommand,PSTR("TA"))==0)
                 {
                       /* read TA */
                       CurrentTA=RS_ReadCmdParamUi() & 0x01;          /* only 1 bit lenght for TA field */
                       SyntaxError=0; /* command OK */                         
                       if (CurrentCommand=='S')
                       {
                          /* Store TA in EEPROM */
                          EEP_set_TA(CurrentTA);
                       }                    
                 }   
                 
                 if (strcmp_P(ParamCommand,PSTR("MS"))==0)
                 {
                       /* read MS */
                       TmpParam=RS_ReadCmdParamUi();
                       TmpParam &= 0x01;          /* only 1 bit lenght for MS field */
                       CurrentMS=  TmpParam<< 3;  /* MS should be shifted before used by RDS_GroupGenerate */ 
                       SyntaxError=0; /* command OK */                         
                       if (CurrentCommand=='S')
                       {
                          /* Store TP in EEPROM */
                          EEP_set_MS(TmpParam);
                       }                    
                 }     
                    	    
                 if (strcmp_P(ParamCommand,PSTR("DI"))==0)
                 {
                       /* read DI */
                       TmpParam=RS_ReadCmdParamUi();
                       TmpParam &= 0x0F;          /* only 4 bits lenght for MS field */
                       CurrentDI=  TmpParam;  /* no shift necessary for DI parameter*/ 
                       SyntaxError=0; /* command OK */                         
                       if (CurrentCommand=='S')
                       {
                          /* Store TP in EEPROM */
                          EEP_set_DI(TmpParam);
                       }                    
                 }        	    
                 
                      	    
                /* ------------------------------------------- */

                 /* specific command for EEPROM only */
                 if (CurrentCommand=='S')
                 {
                    /* parameter "PI" */
                    /* -------------- */  
                    if (strcmp_P(ParamCommand,PSTR("PI"))==0)
                    {
                       /* modifying PICode */
                       TmpParam=RS_ReadCmdParamUi();                    
                       /* store PICode in EEPROM */
                       SyntaxError=0; /* command OK */                         
                       EEP_set_PICode(TmpParam); 
                    }  /* Endif PICode processing */
                    
                    /* parameter "NA" */
                    /* -------------- */  
                    if (strcmp_P(ParamCommand,PSTR("NA"))==0)
                    {
                       /* modifying NbAF */
                       IndexParam=RS_ReadCmdParamUi();                    
                       /* store NbAF in EEPROM */
                       SyntaxError=0; /* command OK */                         
                       EEP_set_NbAF(IndexParam); 
                    }  /* Endif NbAF processing */
                    
                    /* parameter "AF" */
                    /* -------------- */  
                    if (strcmp_P(ParamCommand,PSTR("AF"))==0)
                    {
                       /* Reading AF index */
                       IndexParam=RS_ReadCmdParamUi();
                       /* reading AF value */  
                       TmpParam=RS_ReadCmdParamUi();                                                             
                       /* convert frequency in 100KHz to AF format */
                       TmpParam-=RDS_AF_FIRST_AF_FRQ;
                       /* store AF in EEPROM, 1st param =new val, 2nd =AF index */
                       EEP_set_AF(TmpParam,IndexParam); 
                       SyntaxError=0; /* command OK */                         
                    }  /* Endif AF processing */

                    /* parameter "2A" */
                    /* -------------- */  
                    if (strcmp_P(ParamCommand,PSTR("2A"))==0)
                    {
                       /* modifying 2A sending count */
                       IndexParam=RS_ReadCmdParamUi();                    
                       /* store 2A sending count in EEPROM */
                       EEP_set_GroupRate(IndexParam,(RDS_GROUP_TYPE_2A & RDS_GROUP_TYPE_SELECT),(RDS_GROUP_TYPE_2A>>RDS_GROUP_TYPE_SHIFT)); 
                       SyntaxError=0; /* command OK */                         
                    }  /* Endif 2A group rate  processing */
                    
                    if (strcmp_P(ParamCommand,PSTR("ST"))==0)
                    {
                       /* Reading stereo/mono  1=stereo  0=mono*/
                       IndexParam=RS_ReadCmdParamUi();
                       EEP_set_StereoMode(IndexParam);
                       SyntaxError=0; /* command OK */                         
                    }  /* Endif AF processing */                                  

                    
                    
                 }                 
                 
                 
         	}
         	break;
         
        	 /* =========================================== */
         	case 'E':
         	case 'e':
         	{
               /* erase EEPROM , need confirmation */
               PreviousCommand= 'E';
               RS_PrintfStrPGM(PSTR("reset EEPROM ? (Y/N)\r\n"));              
               SyntaxError=0; /* command OK */                         

            }
            break;
        	 /* =========================================== */
            
         	case 'Y':
         	case 'y':
         	case 'O':
         	case 'o':
         	{
               /* erase EEPROM confirmation , test if previous command equal to erase */
               if (PreviousCommand== 'E')
               {
                  RS_PrintfStrPGM(PSTR("STOP RDS (restart: G 1)\r\n"));              
                  
                  RDSCLK_Stop();
                  EEP_SetDefault();
                  RS_PrintfStrPGM(PSTR("EEPROM reset !! \r\n"));              
                  SyntaxError=0; /* command OK */                         
               }
               PreviousCommand=' ';
            }
            break;
        	 /* =========================================== */
         	case 'N':
         	case 'n':
         	{
               /* erase EEPROM cancel , test if previous command equal to erase */
               if (PreviousCommand== 'E')
               {
                  RS_PrintfStrPGM(PSTR("EEPROM reset canceled.\r\n"));              
               }
               SyntaxError=0; /* command OK */                         
               PreviousCommand=' ';
            }
            break;
        	 /* =========================================== */

         	case 'H':
         	case 'h':
         	case '?':
         	{
         		/* test command end */
         		if(RS_ReadCmdChar() == '\r')
         		{
	         		/* Menu aide */
                   RS_PrintfStrPGM(PSTR("RDSCoder V1.0 05/2007\r\n"));
                   RS_PrintfStrPGM(PSTR("reboot board for help message\r\n"));
                   SyntaxError=0; /* command OK */                         

    	       }
               else
               {
	       	       	SyntaxError=1;	
               }        		
         		
         	}
         	break;
         	
        	 /* =========================================== */
         	
   	        case 'G':
         	case 'g':
         	{
                /* generating Start/stop */ 
         		/* read first param */
         		IndexParam=RS_ReadCmdParamUi();
         		if (IndexParam==1)
         		{
                    /* restart coder generating */
                    RDSCLK_Start();
                }
                else
                {
                    /* stop coder */
                    RDSCLK_Stop();
                }
                SyntaxError=0; /* command OK */                         
            }
            break;

        	 /* =========================================== */
//          comment following line for ATmega8 version
// 	        case 'D':
//         	case 'd':
//         	{
//         		/* read first param */
//        		IndexParam=(unsigned char) RS_ReadCmdParamUi();
//         		if ((IndexParam==0) || (IndexParam==1))
//         		{
//                    /* set DumpBuffer for debug purpose */
//                    RDSDumpBuf=IndexParam;
//                }
//                else
//                {
//                    /* other parameter --> dump SRAM */
//                    RS_DebugDumpSRAM();                
//                }                    
//                SyntaxError=0; /* command OK */                         
//           }  /* end case D */
//           break;
        	 /* =========================================== */
		   
           /* case simple return */ 
           case '\r':
           {
              SyntaxError=0; /* command OK */
           }                         
           break;
        	 /* =========================================== */
         	


         	default:
            SyntaxError=1; /* command NOK */                         

         	RS_PrintfStrPGM(PSTR("\r\n\n\n\n\nErreur cmd: "));
         	RS_DebugPrintfc(CurrentCar);   /* display command as a char*/
         	RS_PrintfStrPGM(PSTR(" inconnue"));
        }
        if (SyntaxError <0)
        {
          	RS_PrintfStrPGM(PSTR("\r\nErreur de parametre\r\n "));
        }
        /* read unused  char to end of line */
        while(RS_getNbCmd() !=0)
        {
              CurrentCar=RS_ReadCmdChar();                        
        }

        RS_PrintfStrPGM(PSTR("\r\n>")); /* prompt print */
        
    } /* End test on command receive */
}


/***************************************************/
/***************************************************/
/*           Fonctions de DEBUG                    */
/***************************************************/
/***************************************************/
/***************************************************/

/***************************************************/
/*    RS_DebugPrintfc :                            */
/*    ---------------                              */
/*      Send a char with polling                   */
/*      DEBUG functions                            */
/***************************************************/
void RS_DebugPrintfc( unsigned char data )
{
        /* polling  wait end sending */
        while ( !(UCSRA & (1<<UDRE)));
        UDR= data; 
}

/***************************************************/
/*    RS_DebugPrintfStr:                           */
/*    -----------                                  */
/*      Send a string reading from RAM             */
/*      DEBUG functions                            */
/***************************************************/
void RS_DebugPrintfStr( unsigned char  * StringChar )
{
        /* printf vers la liaison serie avec IT */
	while (*StringChar !=0)
	{
	    RS_DebugPrintfc( *StringChar ); /* emission du caractere */
	    StringChar++; /* incrementation pointeur */
	};
}
/***************************************************/
/*    RS_DebugPrintfUi :                           */
/*    -----------                                  */
/*      Send an unsigned Int converted into :      */
/*      decimal if Format=0 or hexa if Format=1    */
/*      DEBUG functions                            */
/***************************************************/
void RS_DebugPrintfUi( unsigned int  Data, unsigned char Format )
{
    /* unsigned int printf to the serial link with polling method and format : 0=10, 1=hexa */

	
    switch (Format)
    {
      case 1:
       {
       	  RS_DebugPrintfStr(PSTR("0x"));
       	  utoa(Data,RS_TmpConvertString,16);
       }
       break;
      case 0:
      default:
       {
       	  utoa(Data,RS_TmpConvertString,10);
       }
    }
    RS_DebugPrintfStr(RS_TmpConvertString);
}

/***************************************************/
/*    RS_DebugPrintfStrPGM :                       */
/*    -----------------                            */
/*      Send a string reading from flash memory    */
/*      without RAM copy                           */
/*      DEBUG functions                            */
/***************************************************/
void RS_DebugPrintfStrPGM(PGM_P StringChar )
{
        /* printf to the serial link */
        while (pgm_read_byte(StringChar) !='\0')
        {
            RS_DebugPrintfc(pgm_read_byte(StringChar) ); /* char sending */
            StringChar++; /* pointer incrementating  */
        };
}

/***************************************************/
/*    RS_DebugDumpSRAM :                           */
/*    -----------------                            */
/*      dump SRAM in hex format                    */
/***************************************************/
//          comment following line for ATmega8 version
//void RS_DebugDumpSRAM(void)
//{
//   
//       char *  ptcharDump=0;
//       char IndexDump;
//
//       RDSCLK_Stop();
//       
//       while (ptcharDump<((char *)RAMEND))
//       {
//           /* print adress */
//           RS_DebugPrintfUi((unsigned int)ptcharDump,0);
//           RS_DebugPrintfStrPGM(PSTR(" :"));
//           
//           for (IndexDump=0;IndexDump <16;IndexDump++)
//           {
//               /* print separator */
//               RS_DebugPrintfc(' ');
//               /* print value */
//               RS_DebugPrintfUi((*ptcharDump),1);
//               ptcharDump++;
//           }
//              RS_DebugPrintfStrPGM(PSTR("\r\n"));
//       }
//       
//       /* debug function, restart generating RDS by 'G 1' command */
////      RDSCLK_Start();
//
//}



